﻿using AlphaTrust.CreateTransactions.Models;
using IO.Swagger.Api;
using IO.Swagger.Model;
using Microsoft.Extensions.Configuration;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Threading.Tasks;

namespace AlphaTrust.CreateTransactions
{
    internal class Program
    {
        private static readonly Credential Credential;

        static Program()
        {
            var configuration = new ConfigurationBuilder()
                                .SetBasePath(Directory.GetCurrentDirectory())
                                .AddJsonFile("appsettings.json", false)
                                .Build();

            var appSettings = configuration.GetSection("App").Get<AppSettings>();
            if (appSettings == null)
            {
                throw new InvalidOperationException($"The settings file, appsettings.json, is jacked. See {nameof(AppSettings)}");
            }

            Credential = appSettings.Credentials.FirstOrDefault(c => c.Name == appSettings.Default);
            if (Credential == null)
            {
                throw new InvalidOperationException($"Credential not found for {appSettings.Default}. Set 'Default' in the appsettings.json");
            }
        }

        private static async Task<TransactionResponse> CallApi(TransactionCreateRequestModel request)
        {
            try
            {
                var api = new TransactionsApi(Credential.RestUrl);

                var response = await api.CreateTransactionsAsync(request, Credential.ApiKey, Credential.ApiSecret, Credential.ApiUserName, Credential.ApiPassword);
                Console.WriteLine($"Processing time for transaction {response.Transactions[0].ID} is {response.ProcessingTime}");
                return new TransactionResponse
                       {
                           Elapsed = GetElapsed(response.ProcessingTime),
                           Success = response.IsSuccessful.GetValueOrDefault(false),
                           TransactionID = response.Transactions[0].ID.GetValueOrDefault(0)
                       };
            }
            catch (Exception exception)
            {
                Console.WriteLine($"{exception.Message}");
                throw;
            }
        }

        private static TimeSpan GetElapsed(string value)
        {
            var datetime = DateTime.ParseExact(value.Substring(0,12), "HH:mm:ss.fff", null);
            return TimeSpan.FromTicks(datetime.Ticks);
        }

        private static TransactionCreateRequestModel GetRequest()
        {
            // create the signature task
            var task = GetTask();

            // assign the Task to the Participant
            var participant = new ParticipantInsertModel
                              {
                                  FullName = "John Smith",
                                  EmailAddress = Credential.SignerEmailAddress,
                                  SendRequestViaEmail = true,
                                  Tasks = new List<object> { task }
                              };

            // if you have more than one Participant, you can add them all into the array here
            var participants = new List<object> { participant };

            // attach the document bytes to the document source
            var assembly = Assembly.GetExecutingAssembly();
            var bytes = assembly.ReadAllBytes("1-page.pdf");

            var source = new SourceModel
                         {
                             FileBytes = bytes
                         };

            // assign the Task to the Document
            var document = new DocumentInsertModel(0, "My Test Document", source, tasks: new List<object> { task });

            // if you have more than one Document, you can add them all into the array here
            var documents = new List<object> { document };

            var transaction = new TransactionCreateModel(Credential.ProntoID, true, "My First Transaction Test", documents: documents, participants: participants);

            var transactions = new List<object> { transaction };

            var request = new TransactionCreateRequestModel(transactions)
                          {
                              EnableLogging = true
                          };

            return request;
        }

        private static SignatureBlockModel GetSignatureBlock()
        {
            // define the how the signature is applied on the document
            var placement = GetSignaturePlacement();

            var signature = new SignatureModel(placement)
                            {
                                ApplicationType = SignatureModel.ApplicationTypeEnum.FontStamped,
                                IsBold = true,
                                IsItalic = true
                            };

            var signatureBlock = new SignatureBlockModel(signature);
            return signatureBlock;
        }

        private static PdfSignaturePlacementModel GetSignaturePlacement()
        {
            // define where in the document the signature is placed
            var placement = new PdfSignaturePlacementModel
                            {
                                PageCoordinatesPageNumber = 1,
                                XOffset = 20,
                                YOffset = 20
                            };
            return placement;
        }

        private static TaskInsertModel GetTask()
        {
            // define what type of task to use in the document
            var signatureBlock = GetSignatureBlock();
            var task = new TaskInsertModel
                       {
                           Type = TaskInsertModel.TypeEnum.Signature,
                           SignatureBlock = signatureBlock
                       };

            return task;
        }

        private static async Task Main()
        {
            var request = GetRequest();
            _ = await CallApi(request);

            Console.WriteLine("Press any key to continue...");
            Console.ReadKey();
        }
    }
}